package com.usky.dxtop.service.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.dxtop.common.utils.http.HttpUtils;
import com.usky.dxtop.model.Dept;
import com.usky.dxtop.model.MsgLog;
import com.usky.dxtop.model.Staff;
import com.usky.dxtop.service.DeptService;
import com.usky.dxtop.service.MsgLogService;
import com.usky.dxtop.service.StaffService;
import com.usky.dxtop.service.api.SmApi;
import com.usky.dxtop.service.config.rabbitmq.face.FaceConsumeConfig;
import com.usky.dxtop.service.config.rabbitmq.group.GroupConsumeConfig;
import com.usky.dxtop.service.config.rabbitmq.profile.ProfileConsumeConfig;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.function.Function;

/**
 * 世贸定时任务
 *
 * @author yq
 * @date 2021/9/28 14:47
 */
@Component("smJob")
@Slf4j
public class SmJob {

    @Autowired
    private StaffService staffService;

    @Autowired
    private DeptService deptService;

    @Autowired
    @Qualifier(ProfileConsumeConfig.TEMPLATE)
    private RabbitTemplate profileRabbitTemplate;


    @Autowired
    @Qualifier(FaceConsumeConfig.TEMPLATE)
    private RabbitTemplate faceRabbitTemplate;

    @Autowired
    @Qualifier(GroupConsumeConfig.TEMPLATE)
    private RabbitTemplate groupRabbitTemplate;


    @Autowired
    private MsgLogService msgLogService;

//    @Scheduled(cron = "0 0 0 */1 * ?")
    public void personExecute(String param) {
        PersonParam personParam = JSON.parseObject(param, PersonParam.class);
        personApi(personParam);
        log.info("smJob-person 定时任务完成");
    }

    public void groupExecute(String param) {
        GroupParam groupParam = JSON.parseObject(param, GroupParam.class);
        groupApi(groupParam);
        log.info("smJob-group 定时任务完成");
    }


    public void  personApi(PersonParam personParam){
        TreeMap<String, Object> treeMap;
        if (null != personParam){
            treeMap = SmApi.generatePersonParam(personParam.getId(),
                    personParam.getDeptIds(), personParam.getCreateTime(), personParam.blurry);
        }else {
            treeMap = SmApi.generatePersonParam(null,
                    null, null, null);
        }
        common(SmApi.PERSON_URL,treeMap,staffList -> {
            List<Staff> list = staffList.toJavaList(Staff.class);
            list.forEach(date -> {
                LambdaQueryWrapper<Staff> queryWrapper = Wrappers.lambdaQuery();
                queryWrapper.eq(Staff::getId,date.getId());
                Staff staff = staffService.getOne(queryWrapper);
                int type;
                if (null == staff){
                    type = 1;
                    staffService.save(date);
                }else {
                    type = 2;
                    date.setSId(staff.getSId());
                    staffService.updateById(date);
                }
                //发送人员信息
                personSendMessage(date,type);
                //发送人脸信息
                faceSendMessage(date);
            });
            return list.isEmpty();
        });
    }

    public void groupApi(GroupParam groupParam){
        TreeMap<String, Object> treeMap;
        if (null != groupParam){
            treeMap = SmApi.generateDeptParam(groupParam.getCreateTime(),
                    groupParam.getName(),groupParam.getPid(),groupParam.getEnable());
        }else {
            treeMap = SmApi.generateDeptParam(null,
                    null,null,null);
        }
        common(SmApi.DEPT_URL,treeMap,staffList -> {
            List<Dept> list = staffList.toJavaList(Dept.class);
            list.forEach(date -> {
                LambdaQueryWrapper<Dept> queryWrapper = Wrappers.lambdaQuery();
                queryWrapper.eq(Dept::getId,date.getId());
                Dept dept = deptService.getOne(queryWrapper);
                if (null == dept){
                    deptService.save(date);
                }
                deptSendMessage(date);
            });
            return list.isEmpty();
        });
    }

    public void common(String url, TreeMap<String,Object> param, Function<JSONArray,Boolean> function){
        boolean isNext = true;
        int page = 1;
        int size = 100;
        while (isNext) {
            try {
                String personUrl = String.format("%s?page=%s&size=%s", url, page, size);
                String result = HttpUtils.sendPost(personUrl, JSONObject.toJSONString(param), SmApi.sign(param));
                JSONObject jsonObject = JSONObject.parseObject(result);
                if ("200".equals(jsonObject.get("status").toString())) {
                    JSONObject data = jsonObject.getJSONObject("data");
                    JSONArray content = data.getJSONArray("content");
                    if (content != null && content.size() > 0) {
                        if (function.apply(content)) {
                            isNext = false;
                        } else {
                            page++;
                        }
                    } else {
                        isNext = false;
                    }
                } else {
                    break;
                }
            } catch (Exception e) {
                log.info("smJob异常" + e);
                break;
            }
        }
    }

    /**
     * 人员消息
     * @param staff
     */
    private void personSendMessage(Staff staff,Integer type){
        Map<String,Object> map = new HashMap<>();
        map.put("type",type);
        map.put("card_type",1);
        map.put("card",staff.getCard());
        map.put("name",staff.getName());
        map.put("mob",staff.getId());
        map.put("groupid",staff.getPid());
        map.put("tel","");
        map.put("level_name","");
        MsgLog msgLog = new MsgLog();
        msgLog.setBusinessId(staff.getSId());
        msgLog.setExchange(ProfileConsumeConfig.NAME);
        msgLog.setRoutingKey(ProfileConsumeConfig.NAME);
        msgLogService.save(msgLog);
        map.put("seq",msgLog.getId());
        msgLog.setMsg(JSON.toJSONString(map));
        msgLogService.updateById(msgLog);
        CorrelationData correlationData = new CorrelationData(msgLog.getId().toString());
        profileRabbitTemplate.convertAndSend(msgLog.getExchange(), msgLog.getRoutingKey(), msgLog.getMsg(),correlationData);
    }

    /**
     * 部门消息
     * @param dept
     */
    private void deptSendMessage(Dept dept){
        Map<String,Object> map = new HashMap<>();
        map.put("parentid",dept.getPid());
        map.put("id",dept.getId());
        map.put("name",dept.getName());
        MsgLog msgLog = new MsgLog();
        msgLog.setBusinessId(dept.getDId());
        msgLog.setExchange(GroupConsumeConfig.NAME);
        msgLog.setRoutingKey(GroupConsumeConfig.NAME);
        msgLogService.save(msgLog);
        map.put("seq",msgLog.getId());
        msgLog.setMsg(JSON.toJSONString(map));
        msgLogService.updateById(msgLog);
        CorrelationData correlationData = new CorrelationData(msgLog.getId().toString());
        groupRabbitTemplate.convertAndSend(msgLog.getExchange(), msgLog.getRoutingKey(), msgLog.getMsg(),correlationData);
    }

    /**
     * 人脸
     * @param staff
     */
    private void faceSendMessage(Staff staff){
        Map<String,Object> map = new HashMap<>();
        map.put("mob",staff.getId());
        map.put("image",staff.getFace());
        faceRabbitTemplate.convertAndSend(FaceConsumeConfig.NAME, FaceConsumeConfig.NAME, map);
        MsgLog msgLog = new MsgLog();
        msgLog.setBusinessId(staff.getSId());
        msgLog.setExchange(FaceConsumeConfig.NAME);
        msgLog.setRoutingKey(FaceConsumeConfig.NAME);
        msgLogService.save(msgLog);
        map.put("seq",msgLog.getId());
        msgLog.setMsg(JSON.toJSONString(map));
        msgLogService.updateById(msgLog);
        CorrelationData correlationData = new CorrelationData(msgLog.getId().toString());
        faceRabbitTemplate.convertAndSend(msgLog.getExchange(), msgLog.getRoutingKey(), msgLog.getMsg(),correlationData);
    }

    @Data
    public static class PersonParam {
        private String id;
        private List<String> deptIds;
        private List<Date> createTime;
        private String blurry;
    }


    @Data
    public static class GroupParam {
        private String name;
        private Boolean enable;
        private List<Date> createTime;
        private String pid;
    }
}
